import asyncio
from _typeshed import Incomplete

async def expect_async(expecter, timeout: Incomplete | None = None): ...
async def repl_run_command_async(repl, cmdlines, timeout: int = -1): ...

class PatternWaiter(asyncio.Protocol):
    transport: Incomplete
    expecter: Incomplete
    fut: Incomplete
    def set_expecter(self, expecter) -> None: ...
    def found(self, result) -> None: ...
    def error(self, exc) -> None: ...
    def connection_made(self, transport) -> None: ...
    def data_received(self, data) -> None: ...
    def eof_received(self) -> None: ...
    def connection_lost(self, exc) -> None: ...



from __future__ import annotations

from typing import Any

from pkg_resources import (
    DefaultProvider,
    EggMetadata,
    EggProvider,
    EmptyProvider,
    FileMetadata,
    IMetadataProvider,
    IResourceProvider,
    NullProvider,
    PathMetadata,
    ZipProvider,
)
from setuptools.command.editable_wheel import EditableStrategy, _LinkTree, _StaticPth, _TopLevelFinder
from setuptools.config.expand import EnsurePackagesDiscovered
from setuptools.config.pyprojecttoml import _EnsurePackagesDiscovered

# We don't care about the __init__ methods, only about if an instance respects the Protocol
_: Any = object()

# Test IMetadataProvider Protocol implementers
metadata_provider: IMetadataProvider
metadata_provider = NullProvider(_)
metadata_provider = EggProvider(_)
metadata_provider = EmptyProvider()
metadata_provider = DefaultProvider(_)
metadata_provider = ZipProvider(_)
metadata_provider = FileMetadata(_)
metadata_provider = PathMetadata(_, _)
metadata_provider = EggMetadata(_)

# Test IResourceProvider Protocol implementers
resource_provider: IResourceProvider
resource_provider = NullProvider(_)
resource_provider = EggProvider(_)
resource_provider = EmptyProvider()
resource_provider = DefaultProvider(_)
resource_provider = ZipProvider(_)
resource_provider = FileMetadata(_)
resource_provider = PathMetadata(_, _)
resource_provider = EggMetadata(_)


# Test EditableStrategy Protocol implementers
editable_strategy: EditableStrategy
editable_strategy = _StaticPth(_, _, _)
editable_strategy = _LinkTree(_, _, _, _)
editable_strategy = _TopLevelFinder(_, _)
# Not EditableStrategy due to incompatible __call__ method
editable_strategy = EnsurePackagesDiscovered(_)  # type: ignore
editable_strategy = _EnsurePackagesDiscovered(_, _, _)  # type: ignore



from _typeshed import Incomplete

def is_tournament(G): ...
def hamiltonian_path(G): ...
def random_tournament(n, seed: Incomplete | None = None): ...
def score_sequence(G): ...
def is_reachable(G, s, t): ...
def is_strongly_connected(G): ...



from _typeshed import Incomplete
from collections.abc import Callable, Generator
from typing import Any

from influxdb_client import Dialect
from influxdb_client.client._base import _BaseQueryApi
from influxdb_client.client.flux_table import CSVIterator, FluxRecord, TableList

class QueryOptions:
    profilers: Incomplete
    profiler_callback: Incomplete
    def __init__(
        self, profilers: list[str] | None = None, profiler_callback: Callable[..., Incomplete] | None = None
    ) -> None: ...

class QueryApi(_BaseQueryApi):
    def __init__(self, influxdb_client, query_options=...) -> None: ...
    def query_csv(
        self,
        query: str,
        org: Incomplete | None = None,
        dialect: Dialect = ...,
        params: dict[Incomplete, Incomplete] | None = None,
    ) -> CSVIterator: ...
    def query_raw(
        self, query: str, org: Incomplete | None = None, dialect=..., params: dict[Incomplete, Incomplete] | None = None
    ): ...
    def query(
        self, query: str, org: Incomplete | None = None, params: dict[Incomplete, Incomplete] | None = None
    ) -> TableList: ...
    def query_stream(
        self, query: str, org: Incomplete | None = None, params: dict[Incomplete, Incomplete] | None = None
    ) -> Generator[FluxRecord, Any, None]: ...
    def query_data_frame(
        self,
        query: str,
        org: Incomplete | None = None,
        data_frame_index: list[str] | None = None,
        params: dict[Incomplete, Incomplete] | None = None,
    ): ...
    def query_data_frame_stream(
        self,
        query: str,
        org: Incomplete | None = None,
        data_frame_index: list[str] | None = None,
        params: dict[Incomplete, Incomplete] | None = None,
    ): ...
    def __del__(self) -> None: ...



from _typeshed import Incomplete

def projected_graph(B, nodes, multigraph: bool = False): ...
def weighted_projected_graph(B, nodes, ratio: bool = False): ...
def collaboration_weighted_projected_graph(B, nodes): ...
def overlap_weighted_projected_graph(B, nodes, jaccard: bool = True): ...
def generic_weighted_projected_graph(B, nodes, weight_function: Incomplete | None = None): ...



from _typeshed import Incomplete
from collections.abc import Iterable
from datetime import date, datetime, timedelta
from time import _TimeTuple, struct_time
from typing import Any, Generic, NamedTuple, TypeVar, overload

_GetterReturnType = TypeVar("_GetterReturnType")
_SetterValueType = TypeVar("_SetterValueType")

class _AsymmetricProperty(Generic[_GetterReturnType, _SetterValueType]):
    @overload
    def __get__(self, __obj: None, __type: type | None = ...) -> property: ...
    @overload
    def __get__(self, __obj: Any, __type: type | None = ...) -> _GetterReturnType: ...
    def __set__(self, __obj: Any, __value: _SetterValueType) -> None: ...

class _AsymmetricPropertyWithDelete(_AsymmetricProperty[_GetterReturnType, _SetterValueType]):
    def __delete__(self, __obj: Any) -> None: ...

class _StringProperty(_AsymmetricPropertyWithDelete[str | None, str | None]): ...
class _ListProperty(_AsymmetricPropertyWithDelete[tuple[str, ...] | None, Iterable[str] | str | None]): ...
class _DateProperty(
    _AsymmetricPropertyWithDelete[datetime | None, date | datetime | timedelta | _TimeTuple | struct_time | float | str | None]
): ...

def environ_getter(key, default=..., rfc_section: Incomplete | None = None): ...
def environ_decoder(key, default=..., rfc_section: Incomplete | None = None, encattr: Incomplete | None = None): ...
def upath_property(key): ...
def deprecated_property(attr, name, text, version): ...
def header_getter(header: str, rfc_section: str) -> _StringProperty: ...
def converter(prop, parse, serialize, convert_name: Incomplete | None = None): ...
def list_header(header: str, rfc_section: str) -> _ListProperty: ...
def parse_list(value): ...
def serialize_list(value): ...
def converter_date(prop): ...
def date_header(header: str, rfc_section: str) -> _DateProperty: ...
def parse_etag_response(value, strong: bool = False): ...
def serialize_etag_response(value): ...
def serialize_if_range(value): ...
def parse_range(value): ...
def serialize_range(value): ...
def parse_int(value): ...
def parse_int_safe(value): ...

serialize_int = str

def parse_content_range(value): ...
def serialize_content_range(value): ...
def parse_auth_params(params): ...

known_auth_schemes: dict[str, None]

class _authorization(NamedTuple):
    authtype: str
    params: dict[str, str] | str

def parse_auth(val): ...
def serialize_auth(val): ...



from _typeshed import ReadableBuffer

class Error(Exception): ...

def ZSTD_compress(__data: ReadableBuffer, __level: int = ..., __threads: int = ...) -> bytes: ...
def ZSTD_external() -> int: ...
def ZSTD_uncompress(__data: ReadableBuffer) -> bytes: ...
def ZSTD_version() -> str: ...
def ZSTD_version_number() -> int: ...
def ZSTD_threads_count() -> int: ...
def ZSTD_max_threads_count() -> int: ...
def compress(__data: ReadableBuffer, __level: int = ..., __threads: int = ...) -> bytes: ...
def decompress(__data: ReadableBuffer) -> bytes: ...
def dumps(__data: ReadableBuffer, __level: int = ..., __threads: int = ...) -> bytes: ...
def loads(__data: ReadableBuffer) -> bytes: ...
def uncompress(__data: ReadableBuffer) -> bytes: ...
def version() -> str: ...



from _typeshed import IdentityFunction
from collections.abc import Callable, Sequence
from typing import TypeVar

__all__ = ("fifo_cache", "lfu_cache", "lru_cache", "mru_cache", "rr_cache", "ttl_cache")
_T = TypeVar("_T")

def fifo_cache(maxsize: float | None = 128, typed: bool = False) -> IdentityFunction: ...
def lfu_cache(maxsize: float | None = 128, typed: bool = False) -> IdentityFunction: ...
def lru_cache(maxsize: float | None = 128, typed: bool = False) -> IdentityFunction: ...
def mru_cache(maxsize: float | None = 128, typed: bool = False) -> IdentityFunction: ...
def rr_cache(
    maxsize: float | None = 128, choice: Callable[[Sequence[_T]], _T] | None = ..., typed: bool = False
) -> IdentityFunction: ...
def ttl_cache(
    maxsize: float | None = 128, ttl: float = 600, timer: Callable[[], float] = ..., typed: bool = False
) -> IdentityFunction: ...



from _typeshed import Incomplete
from collections.abc import Callable
from os import _Environ

from .spawnbase import SpawnBase, _Logfile

PY3: Incomplete

class spawn(SpawnBase):
    use_native_pty_fork: bool
    STDIN_FILENO: Incomplete
    STDOUT_FILENO: Incomplete
    STDERR_FILENO: Incomplete
    str_last_chars: int
    env: Incomplete
    name: str
    def __init__(
        self,
        command: str,
        args: list[str] = [],
        timeout: float | None = 30,
        maxread: int = 2000,
        searchwindowsize: int | None = None,
        logfile: _Logfile | None = None,
        cwd: str | bytes | None = None,
        env: _Environ[Incomplete] | None = None,
        ignore_sighup: bool = False,
        echo: bool = True,
        preexec_fn: Callable[[Incomplete], Incomplete] | None = None,
        encoding: str | None = None,
        codec_errors: str = "strict",
        dimensions: tuple[int, int] | None = None,
        use_poll: bool = False,
    ) -> None: ...
    child_fd: int
    closed: bool
    def close(self, force: bool = True) -> None: ...
    def isatty(self): ...
    def waitnoecho(self, timeout: int = -1): ...
    def getecho(self): ...
    def setecho(self, state: bool): ...
    def read_nonblocking(self, size: int = 1, timeout: int | None = -1) -> bytes: ...
    def write(self, s: str | bytes) -> None: ...
    def writelines(self, sequence: list[str | bytes]) -> None: ...
    def send(self, s: str | bytes): ...
    def sendline(self, s: str | bytes = ""): ...
    def sendcontrol(self, char: str): ...
    def sendeof(self) -> None: ...
    def sendintr(self) -> None: ...
    @property
    def flag_eof(self) -> bool: ...
    @flag_eof.setter
    def flag_eof(self, value: bool) -> None: ...
    def eof(self) -> bool: ...
    def terminate(self, force: bool = False): ...
    status: int | None
    exitstatus: bool | None
    signalstatus: int | None
    terminated: bool
    def wait(self) -> int: ...
    def isalive(self) -> bool: ...
    def kill(self, sig: int) -> None: ...
    def getwinsize(self) -> tuple[int, int]: ...
    def setwinsize(self, rows, cols) -> None: ...
    def interact(
        self,
        escape_character="\x1d",
        input_filter: Callable[[Incomplete], Incomplete] | None = None,
        output_filter: Callable[[Incomplete], Incomplete] | None = None,
    ) -> None: ...

def spawnu(*args: str, **kwargs: str): ...



from collections.abc import Iterable, Mapping
from re import Pattern
from typing import Any, overload
from typing_extensions import TypeAlias

_Option: TypeAlias = tuple[str, str | None, str]
_GR: TypeAlias = tuple[list[str], OptionDummy]

longopt_pat: str
longopt_re: Pattern[str]
neg_alias_re: Pattern[str]
longopt_xlate: dict[int, int]

class FancyGetopt:
    def __init__(self, option_table: list[_Option] | None = None) -> None: ...
    # TODO kinda wrong, `getopt(object=object())` is invalid
    @overload
    def getopt(self, args: list[str] | None = None) -> _GR: ...
    @overload
    def getopt(self, args: list[str] | None, object: Any) -> list[str]: ...
    def get_option_order(self) -> list[tuple[str, str]]: ...
    def generate_help(self, header: str | None = None) -> list[str]: ...

def fancy_getopt(
    options: list[_Option], negative_opt: Mapping[_Option, _Option], object: Any, args: list[str] | None
) -> list[str] | _GR: ...

WS_TRANS: dict[int, str]

def wrap_text(text: str, width: int) -> list[str]: ...
def translate_longopt(opt: str) -> str: ...

class OptionDummy:
    def __init__(self, options: Iterable[str] = []) -> None: ...



from _typeshed import Incomplete
from io import BytesIO, StringIO

from .exceptions import EOF, TIMEOUT
from .spawnbase import SpawnBase

class searcher_string:
    eof_index: int
    timeout_index: int
    longest_string: int
    def __init__(self, strings) -> None: ...
    match: Incomplete
    start: Incomplete
    end: Incomplete
    def search(self, buffer, freshlen, searchwindowsize: Incomplete | None = None): ...

class searcher_re:
    eof_index: int
    timeout_index: int
    def __init__(self, patterns) -> None: ...
    start: Incomplete
    match: Incomplete
    end: Incomplete
    def search(self, buffer, freshlen: int, searchwindowsize: int | None = None): ...

class Expecter:
    spawn: BytesIO | StringIO
    searcher: searcher_re | searcher_string
    searchwindowsize: int | None
    lookback: searcher_string | searcher_re | int | None
    def __init__(self, spawn: SpawnBase, searcher: searcher_re | searcher_string, searchwindowsize: int = -1) -> None: ...
    def do_search(self, window: str, freshlen: int): ...
    def existing_data(self): ...
    def new_data(self, data: Incomplete): ...
    def eof(self, err: Incomplete | None = None) -> int | EOF: ...
    def timeout(self, err: object | None = None) -> int | TIMEOUT: ...
    def errored(self) -> None: ...
    def expect_loop(self, timeout: int = -1): ...



from _typeshed import Incomplete

string_types: Incomplete

def is_executable_file(path): ...
def which(filename, env: Incomplete | None = None): ...
def split_command_line(command_line): ...
def select_ignore_interrupts(iwtd, owtd, ewtd, timeout: Incomplete | None = None): ...
def poll_ignore_interrupts(fds, timeout: Incomplete | None = None): ...



from _typeshed import Incomplete, Unused

from .ContainerIO import ContainerIO

class TarIO(ContainerIO):
    fh: Incomplete
    def __init__(self, tarfile, file) -> None: ...
    def __enter__(self): ...
    def __exit__(self, *args: Unused) -> None: ...
    def close(self) -> None: ...



from _typeshed import Incomplete

def cut_size(G, S, T: Incomplete | None = None, weight: Incomplete | None = None): ...
def volume(G, S, weight: Incomplete | None = None): ...
def normalized_cut_size(G, S, T: Incomplete | None = None, weight: Incomplete | None = None): ...
def conductance(G, S, T: Incomplete | None = None, weight: Incomplete | None = None): ...
def edge_expansion(G, S, T: Incomplete | None = None, weight: Incomplete | None = None): ...
def mixing_expansion(G, S, T: Incomplete | None = None, weight: Incomplete | None = None): ...
def node_expansion(G, S): ...
def boundary_expansion(G, S): ...



from _typeshed import Incomplete
from typing import Any

from ...client import Pipeline as ClientPipeline
from .commands import JSONCommands

class JSON(JSONCommands):
    MODULE_CALLBACKS: dict[str, Any]
    client: Any
    execute_command: Any
    MODULE_VERSION: Incomplete | None
    def __init__(self, client, version: Incomplete | None = None, decoder=..., encoder=...) -> None: ...
    def pipeline(self, transaction: bool = True, shard_hint: Incomplete | None = None) -> Pipeline: ...

class Pipeline(JSONCommands, ClientPipeline[Incomplete]): ...  # type: ignore[misc]



from _typeshed import Incomplete
from typing import Any

class User:
    type: Any
    id: Any
    display_name: Any
    def __init__(self, parent: Incomplete | None = None, id: str = "", display_name: str = "") -> None: ...
    def startElement(self, name, attrs, connection): ...
    def endElement(self, name, value, connection): ...
    def to_xml(self, element_name: str = "Owner"): ...



from _typeshed import Incomplete

def effective_size(G, nodes: Incomplete | None = None, weight: Incomplete | None = None): ...
def constraint(G, nodes: Incomplete | None = None, weight: Incomplete | None = None): ...
def local_constraint(G, u, v, weight: Incomplete | None = None): ...



from _typeshed import Incomplete

def local_node_connectivity(G, source, target, cutoff: Incomplete | None = None): ...
def node_connectivity(G, s: Incomplete | None = None, t: Incomplete | None = None): ...
def all_pairs_node_connectivity(G, nbunch: Incomplete | None = None, cutoff: Incomplete | None = None): ...



from _typeshed import Incomplete
from collections.abc import Generator

def write_edgelist(G, path, comments: str = "#", delimiter: str = " ", data: bool = True, encoding: str = "utf-8") -> None: ...
def generate_edgelist(G, delimiter: str = " ", data: bool = True) -> Generator[Incomplete, None, None]: ...
def parse_edgelist(
    lines,
    comments: str = "#",
    delimiter: Incomplete | None = None,
    create_using: Incomplete | None = None,
    nodetype: Incomplete | None = None,
    data: bool = True,
): ...
def read_edgelist(
    path,
    comments: str = "#",
    delimiter: Incomplete | None = None,
    create_using: Incomplete | None = None,
    nodetype: Incomplete | None = None,
    data: bool = True,
    edgetype: Incomplete | None = None,
    encoding: str = "utf-8",
): ...



from _typeshed import Incomplete

def node_link_data(
    G,
    attrs: Incomplete | None = None,
    *,
    source: str = "source",
    target: str = "target",
    name: str = "id",
    key: str = "key",
    link: str = "links",
): ...
def node_link_graph(
    data,
    directed: bool = False,
    multigraph: bool = True,
    attrs: Incomplete | None = None,
    *,
    source: str = "source",
    target: str = "target",
    name: str = "id",
    key: str = "key",
    link: str = "links",
): ...



from _typeshed import Incomplete

def laplacian_spectrum(G, weight: str = "weight"): ...
def normalized_laplacian_spectrum(G, weight: str = "weight"): ...
def adjacency_spectrum(G, weight: str = "weight"): ...
def modularity_spectrum(G): ...
def bethe_hessian_spectrum(G, r: Incomplete | None = None): ...



def trophic_levels(G, weight: str = "weight"): ...
def trophic_differences(G, weight: str = "weight"): ...
def trophic_incoherence_parameter(G, weight: str = "weight", cannibalism: bool = False): ...



from collections.abc import Hashable
from typing import TypeVar

from networkx.classes.graph import Graph

_G = TypeVar("_G", bound=Graph[Hashable])

def complement(G): ...
def reverse(G: _G, copy: bool = True) -> _G: ...



from _typeshed import Incomplete

def triangles(G, nodes: Incomplete | None = None): ...
def average_clustering(G, nodes: Incomplete | None = None, weight: Incomplete | None = None, count_zeros: bool = True): ...
def clustering(G, nodes: Incomplete | None = None, weight: Incomplete | None = None): ...
def transitivity(G): ...
def square_clustering(G, nodes: Incomplete | None = None): ...
def generalized_degree(G, nodes: Incomplete | None = None): ...



from _typeshed import Incomplete
from collections.abc import Generator

def connected_components(G) -> Generator[Incomplete, None, None]: ...
def number_connected_components(G): ...
def is_connected(G): ...
def node_connected_component(G, n): ...



from _typeshed import Incomplete

from .spawnbase import SpawnBase, _Logfile

class fdspawn(SpawnBase):
    args: Incomplete
    command: Incomplete
    child_fd: Incomplete
    own_fd: bool
    closed: bool
    name: Incomplete
    use_poll: Incomplete
    def __init__(
        self,
        fd,
        args: Incomplete | None = None,
        timeout: int = 30,
        maxread: int = 2000,
        searchwindowsize: Incomplete | None = None,
        logfile: _Logfile | None = None,
        encoding: Incomplete | None = None,
        codec_errors: str = "strict",
        use_poll: bool = False,
    ) -> None: ...
    def close(self) -> None: ...
    def isalive(self) -> bool: ...
    def terminate(self, force: bool = False) -> None: ...
    def send(self, s: str | bytes) -> bytes: ...
    def sendline(self, s: str | bytes) -> bytes: ...
    def write(self, s) -> None: ...
    def writelines(self, sequence) -> None: ...
    def read_nonblocking(self, size: int = 1, timeout: int | None = -1) -> bytes: ...



from _typeshed import Incomplete
from collections.abc import Generator

def is_biconnected(G): ...
def biconnected_component_edges(G) -> Generator[Incomplete, Incomplete, None]: ...
def biconnected_components(G) -> Generator[Incomplete, None, None]: ...
def articulation_points(G) -> Generator[Incomplete, None, None]: ...



from _typeshed import Incomplete

def current_flow_closeness_centrality(G, weight: Incomplete | None = None, dtype=..., solver: str = "lu"): ...

information_centrality = current_flow_closeness_centrality



from collections.abc import Callable, Mapping, Sequence
from typing import Any, ClassVar, Literal, Protocol
from typing_extensions import Self
from xml.etree.ElementTree import Element

from . import blockparser, inlinepatterns, postprocessors, preprocessors, treeprocessors
from .extensions import Extension
from .util import HtmlStash, Registry

# TODO: The following protocols can be replaced by their counterparts from
# codecs, once they have been propagated to all type checkers.
class _WritableStream(Protocol):
    def write(self, __data: bytes) -> object: ...
    def seek(self, __offset: int, __whence: int) -> object: ...
    def close(self) -> object: ...

class _ReadableStream(Protocol):
    def read(self, __size: int = ...) -> bytes: ...
    def seek(self, __offset: int, __whence: int) -> object: ...
    def close(self) -> object: ...

class Markdown:
    preprocessors: Registry[preprocessors.Preprocessor]
    inlinePatterns: Registry[inlinepatterns.Pattern]
    treeprocessors: Registry[treeprocessors.Treeprocessor]
    postprocessors: Registry[postprocessors.Postprocessor]
    parser: blockparser.BlockParser
    htmlStash: HtmlStash
    output_formats: ClassVar[dict[Literal["xhtml", "html"], Callable[[Element], str]]]
    output_format: Literal["xhtml", "html"]
    serializer: Callable[[Element], str]
    tab_length: int
    block_level_elements: list[str]
    registeredExtensions: list[Extension]
    ESCAPED_CHARS: list[str]
    doc_tag: ClassVar[str]
    stripTopLevelTags: bool
    def __init__(
        self,
        *,
        extensions: Sequence[str | Extension] | None = ...,
        extension_configs: Mapping[str, Mapping[str, Any]] | None = ...,
        output_format: Literal["xhtml", "html"] | None = ...,
        tab_length: int | None = ...,
    ) -> None: ...
    def build_parser(self) -> Self: ...
    def registerExtensions(self, extensions: Sequence[Extension | str], configs: Mapping[str, dict[str, Any]]) -> Self: ...
    def build_extension(self, ext_name: str, configs: Mapping[str, Any]) -> Extension: ...
    def registerExtension(self, extension: Extension) -> Self: ...
    def reset(self) -> Self: ...
    def set_output_format(self, format: Literal["xhtml", "html"]) -> Self: ...
    def is_block_level(self, tag: object) -> bool: ...
    def convert(self, source: str) -> str: ...
    def convertFile(
        self, input: str | _ReadableStream | None = None, output: str | _WritableStream | None = None, encoding: str | None = None
    ) -> Self: ...

def markdown(
    text: str,
    *,
    extensions: Sequence[str | Extension] | None = ...,
    extension_configs: Mapping[str, Mapping[str, Any]] | None = ...,
    output_format: Literal["xhtml", "html"] | None = ...,
    tab_length: int | None = ...,
) -> str: ...
def markdownFromFile(
    *,
    input: str | _ReadableStream | None = ...,
    output: str | _WritableStream | None = ...,
    encoding: str | None = ...,
    extensions: Sequence[str | Extension] | None = ...,
    extension_configs: Mapping[str, Mapping[str, Any]] | None = ...,
    output_format: Literal["xhtml", "html"] | None = ...,
    tab_length: int | None = ...,
) -> None: ...



from _typeshed import Incomplete

def current_flow_betweenness_centrality_subset(
    G, sources, targets, normalized: bool = True, weight: Incomplete | None = None, dtype=..., solver: str = "lu"
): ...
def edge_current_flow_betweenness_centrality_subset(
    G, sources, targets, normalized: bool = True, weight: Incomplete | None = None, dtype=..., solver: str = "lu"
): ...



from _typeshed import Incomplete
from collections.abc import Iterable
from typing import Any, Literal
from typing_extensions import TypeAlias

from matplotlib.axes import Axes
from matplotlib.colors import Colormap

from .axisgrid import FacetGrid
from .utils import _ErrorBar, _Estimator, _Legend, _Palette, _Seed

__all__ = ["relplot", "scatterplot", "lineplot"]

_Sizes: TypeAlias = list[float] | dict[str, float] | tuple[float, float]

def lineplot(
    data: Incomplete | None = None,
    *,
    x: Incomplete | None = None,
    y: Incomplete | None = None,
    hue: Incomplete | None = None,
    size: Incomplete | None = None,
    style: Incomplete | None = None,
    units: Incomplete | None = None,
    palette: _Palette | Colormap | None = None,
    hue_order: Iterable[Any] | None = None,
    hue_norm: Incomplete | None = None,
    sizes: _Sizes | None = None,
    size_order: Iterable[Any] | None = None,
    size_norm: Incomplete | None = None,
    dashes: bool | list[Incomplete] | dict[str, Incomplete] = True,
    markers: Incomplete | None = None,
    style_order: Iterable[Any] | None = None,
    estimator: _Estimator | None = "mean",
    errorbar: _ErrorBar | None = ("ci", 95),
    n_boot: int = 1000,
    seed: _Seed | None = None,
    orient: Literal["x", "y"] = "x",
    sort: bool = True,
    err_style: Literal["band", "bars"] = "band",
    err_kws: dict[str, Any] | None = None,
    legend: _Legend = "auto",
    ci: str | int | None = "deprecated",  # deprecated
    ax: Axes | None = None,
    **kwargs: Any,
) -> Axes: ...
def scatterplot(
    data: Incomplete | None = None,
    *,
    x: Incomplete | None = None,
    y: Incomplete | None = None,
    hue: Incomplete | None = None,
    size: Incomplete | None = None,
    style: Incomplete | None = None,
    palette: _Palette | Colormap | None = None,
    hue_order: Iterable[Any] | None = None,
    hue_norm: Incomplete | None = None,
    sizes: _Sizes | None = None,
    size_order: Iterable[Any] | None = None,
    size_norm: Incomplete | None = None,
    markers: Incomplete = True,
    style_order: Iterable[Any] | None = None,
    legend: _Legend = "auto",
    ax: Axes | None = None,
    **kwargs: Any,
) -> Axes: ...
def relplot(
    data: Incomplete | None = None,
    *,
    x: Incomplete | None = None,
    y: Incomplete | None = None,
    hue: Incomplete | None = None,
    size: Incomplete | None = None,
    style: Incomplete | None = None,
    units: Incomplete | None = None,
    row: Incomplete | None = None,
    col: Incomplete | None = None,
    col_wrap: int | None = None,
    row_order: Iterable[Any] | None = None,
    col_order: Iterable[Any] | None = None,
    palette: _Palette | Colormap | None = None,
    hue_order: Iterable[Any] | None = None,
    hue_norm: Incomplete | None = None,
    sizes: _Sizes | None = None,
    size_order: Iterable[Any] | None = None,
    size_norm: Incomplete | None = None,
    markers: Incomplete | None = None,
    dashes: Incomplete | None = None,
    style_order: Iterable[Any] | None = None,
    legend: _Legend = "auto",
    kind: Literal["scatter", "line"] = "scatter",
    height: float = 5,
    aspect: float = 1,
    facet_kws: dict[str, Any] | None = None,
    **kwargs: Any,
) -> FacetGrid: ...



from _typeshed import Incomplete
from collections.abc import Generator, Sequence

from networkx.classes.graph import Graph, _Node

__all__ = ["all_simple_paths", "is_simple_path", "shortest_simple_paths", "all_simple_edge_paths"]

def is_simple_path(G: Graph[_Node], nodes: Sequence[_Node]): ...
def all_simple_paths(
    G: Graph[_Node], source: _Node, target: _Node, cutoff: Incomplete | None = None
) -> Generator[list[_Node], None, None]: ...
def all_simple_edge_paths(
    G: Graph[_Node], source: _Node, target: _Node, cutoff: Incomplete | None = None
) -> Generator[list[_Node] | list[tuple[_Node, _Node]], None, list[_Node] | None]: ...
def shortest_simple_paths(
    G: Graph[_Node], source: _Node, target: _Node, weight: Incomplete | None = None
) -> Generator[list[_Node], None, None]: ...

class PathBuffer:
    paths: Incomplete
    sortedpaths: Incomplete
    counter: Incomplete
    def __init__(self) -> None: ...
    def __len__(self): ...
    def push(self, cost, path) -> None: ...
    def pop(self): ...



from _typeshed import Incomplete
from collections.abc import Callable

__all__ = ["_dispatch", "_mark_tests"]

class PluginInfo:
    def __init__(self) -> None: ...
    def __bool__(self) -> bool: ...
    @property
    def items(self): ...
    def __contains__(self, name) -> bool: ...
    def __getitem__(self, name): ...

def _dispatch(func: Incomplete | None = None, *, name: Incomplete | None = None) -> Callable[..., Incomplete]: ...
def _mark_tests(items) -> None: ...



from _typeshed import Incomplete

def color(G): ...
def is_bipartite(G): ...
def is_bipartite_node_set(G, nodes): ...
def sets(G, top_nodes: Incomplete | None = None): ...
def density(B, nodes): ...
def degrees(B, nodes, weight: Incomplete | None = None): ...



from _typeshed import HasFileno, OptExcInfo, ReadOnlyBuffer
from _typeshed.wsgi import WSGIApplication
from collections.abc import Callable
from types import ModuleType
from typing import Any, Literal, Protocol, final, overload
from typing_extensions import Self, TypeAlias

import uwsgidecorators

_TrueOrNone: TypeAlias = Literal[True] | None

class _RPCCallable(Protocol):
    def __call__(self, *args: bytes) -> bytes | None: ...

# FIXME: Technically we know the exact layout of _AppsDict and _WorkerDict
#        but TypedDict does not support bytes keys, so for now we use type
#        aliases to a more generic dict
_WorkerDict: TypeAlias = dict[bytes, Any]

SPOOL_IGNORE: Literal[0]
SPOOL_OK: Literal[-2]
SPOOL_RETRY: Literal[-1]
applications: dict[str, WSGIApplication | str] | None
buffer_size: int
cores: int
has_threads: int
hostname: bytes
is_a_reload: bool
loop: bytes | None
magic_table: dict[bytes, bytes]
numproc: int
opt: dict[str, bytes | Literal[True] | list[bytes | Literal[True]]]
sockets: list[int]
started_on: int
unbit: _TrueOrNone
version: bytes
version_info: tuple[int, int, int, int, bytes]
spoolers: tuple[bytes, ...]
queue_size: int

decorators = uwsgidecorators
spooler = uwsgidecorators.manage_spool_request
post_fork_hook = uwsgidecorators.postfork_chain_hook

@final
class SymbolsImporter:
    def find_module(self, __fullname: str) -> Self | None: ...
    def load_module(self, __fullname: str) -> ModuleType | None: ...

@final
class SymbolsZipImporter:
    def __init__(self, __name: str) -> None: ...
    def find_module(self, __fullname: str) -> Self | None: ...
    def load_module(self, __fullname: str) -> ModuleType | None: ...

@final
class ZipImporter:
    def __init__(self, __name: str) -> None: ...
    def find_module(self, __fullname: str) -> Self | None: ...
    def load_module(self, __fullname: str) -> ModuleType | None: ...

def accepting(__accepting: bool = True) -> None: ...
def add_cron(__signum: int, __minute: int, __hour: int, __day: int, __month: int, __weekday: int) -> Literal[True]: ...
def add_file_monitor(__signum: int, __filename: str) -> None: ...
def add_rb_timer(__signum: int, __seconds: int, __iterations: int = 0) -> None: ...
def add_timer(__signum: int, __seconds: int) -> None: ...
def add_var(__key: bytes | str, __val: bytes | str) -> Literal[True]: ...
def alarm(__alarm: str, __msg: bytes | ReadOnlyBuffer | str) -> None: ...
def async_connect(__socket_name: str) -> int: ...
def async_sleep(__timeout: float) -> Literal[b""]: ...
def cache_clear(__cache_name: str = ...) -> _TrueOrNone: ...
def cache_dec(__key: str | bytes, __decrement: int = 1, __expires: int = 0, __cache_name: str = ...) -> _TrueOrNone: ...
def cache_del(__key: str | bytes, __cache_name: str = ...) -> _TrueOrNone: ...
def cache_div(__key: str | bytes, __divisor: int = 2, __expires: int = 0, __cache_name: str = ...) -> _TrueOrNone: ...
def cache_exists(__key: str | bytes, __cache_name: str = ...) -> _TrueOrNone: ...
def cache_get(__key: str | bytes, __cache_name: str = ...) -> bytes | None: ...
def cache_inc(__key: str | bytes, __increment: int = 1, __expires: int = 0, __cache_name: str = ...) -> _TrueOrNone: ...
def cache_keys(__cache_name: str = ...) -> list[bytes]: ...
def cache_mul(__key: str | bytes, __factor: int = 2, __expires: int = 0, __cache_name: str = ...) -> _TrueOrNone: ...
def cache_num(__key: str | bytes, __cache_name: str = ...) -> int: ...
def cache_set(
    __key: str | bytes, __value: str | bytes | ReadOnlyBuffer, __expires: int = 0, __cache_name: str = ...
) -> _TrueOrNone: ...
def cache_update(
    __key: str | bytes, __value: str | bytes | ReadOnlyBuffer, __expires: int = 0, __cache_name: str = ...
) -> _TrueOrNone: ...
def queue_get(__index: int) -> bytes | None: ...
def queue_set(__index: int, __message: str | bytes | ReadOnlyBuffer) -> _TrueOrNone: ...
@overload
def queue_last(__num: Literal[0] = 0) -> bytes | None: ...  # type: ignore[overload-overlap]
@overload
def queue_last(__num: int) -> list[bytes | None]: ...
def queue_push(__message: str | bytes | ReadOnlyBuffer) -> _TrueOrNone: ...
def queue_pull() -> bytes | None: ...
def queue_pop() -> bytes | None: ...
def queue_slot() -> int: ...
def queue_pull_slot() -> int: ...
def snmp_set_community(__snmp_community: str) -> Literal[True]: ...
def snmp_set_counter32(__oid_num: int, __value: int) -> _TrueOrNone: ...
def snmp_set_counter64(__oid_num: int, __value: int) -> _TrueOrNone: ...
def snmp_set_gauge(__oid_num: int, __value: int) -> _TrueOrNone: ...
def snmp_incr_counter32(__oid_num: int, __increment: int) -> _TrueOrNone: ...
def snmp_incr_counter64(__oid_num: int, __increment: int) -> _TrueOrNone: ...
def snmp_incr_gauge(__oid_num: int, __increment: int) -> _TrueOrNone: ...
def snmp_decr_counter32(__oid_num: int, __decrement: int) -> _TrueOrNone: ...
def snmp_decr_counter64(__oid_num: int, __decrement: int) -> _TrueOrNone: ...
def snmp_decr_gauge(__oid_num: int, __decrement: int) -> _TrueOrNone: ...
@overload
def send_to_spooler(__mesage_dict: dict[bytes, bytes]) -> bytes | None: ...
@overload
def send_to_spooler(
    *, spooler: bytes = ..., priority: bytes = ..., at: bytes = ..., body: bytes = ..., **kwargs: bytes
) -> bytes | None: ...

spool = send_to_spooler

def set_spooler_frequency(__frequency: int) -> Literal[True]: ...
def spooler_jobs() -> list[bytes]: ...
def spooler_pid() -> int: ...
def spooler_pids() -> list[int]: ...
def spooler_get_task(__task_path: str) -> dict[bytes, bytes] | None: ...
def call(__rpc_name: str, *args: bytes) -> bytes | None: ...
def chunked_read(__timeout: int = 0) -> bytes: ...
def chunked_read_nb() -> bytes: ...
def cl() -> int: ...
def close(__fd: int) -> None: ...
def connect(__socket_name: str, timeout: int = 0) -> int: ...
def connection_fd() -> int: ...
def disconnect() -> None: ...
def embedded_data(__name: str) -> bytes: ...
def extract(__name: str) -> bytes | None: ...
def farm_get_msg() -> bytes | None: ...
def farm_msg(__farm_name: str, __message: str | bytes | ReadOnlyBuffer) -> None: ...
def get_logvar(__key: str | bytes) -> bytes | None: ...
def green_schedule() -> Literal[True]: ...
def i_am_the_lord(__legion_name: str) -> bool: ...
def i_am_the_spooler() -> _TrueOrNone: ...
def in_farm(__farm_name: str = ...) -> _TrueOrNone: ...
def is_connected(__fd: int) -> bool: ...
def is_locked(__lock_num: int = 0) -> bool: ...
def listen_queue(__id: int = 0) -> int: ...
def lock(__lock_num: int = 0) -> None: ...
def log(__logline: str) -> Literal[True]: ...
def log_this_request() -> None: ...
def logsize() -> int: ...
def lord_scroll(__legion_name: str) -> bytes | None: ...
def masterpid() -> int: ...
def mem() -> tuple[int, int]: ...
def metric_dec(__key: str, __decrement: int = 1) -> _TrueOrNone: ...
def metric_div(__key: str, __divisor: int = 1) -> _TrueOrNone: ...
def metric_get(__key: str) -> int: ...
def metric_inc(__key: str, __increment: int = 1) -> _TrueOrNone: ...
def metric_mul(__key: str, __factor: int = 1) -> _TrueOrNone: ...
def metric_set(__key: str, __value: int = 1) -> _TrueOrNone: ...
def metric_set_max(__key: str, __value: int = 1) -> _TrueOrNone: ...
def metric_set_min(__key: str, __value: int = 1) -> _TrueOrNone: ...
def micros() -> int: ...
def mule_get_msg(signals: bool = True, farms: bool = True, buffer_size: int = 65536, timeout: int = -1) -> bytes: ...
def mule_id() -> int: ...
@overload
def mule_msg(__mesage: str | bytes | ReadOnlyBuffer) -> bool: ...
@overload
def mule_msg(__mesage: str | bytes | ReadOnlyBuffer, __mule_id: int) -> bool: ...
@overload
def mule_msg(__mesage: str | bytes | ReadOnlyBuffer, __farm_name: str) -> bool: ...
def offload(__filename: str, __len: int = 0) -> Literal[b""]: ...
def parsefile(__filename: str) -> dict[bytes, bytes] | None: ...
def ready() -> _TrueOrNone: ...
def ready_fd() -> int: ...
def recv(__fd: int, __max_size: int = 4096) -> bytes | None: ...
@overload
def register_rpc(__name: str, __func: Callable[[], bytes | None]) -> Literal[True]: ...
@overload
def register_rpc(__name: str, __func: Callable[[bytes], bytes | None], arg_count: Literal[1]) -> Literal[True]: ...
@overload
def register_rpc(__name: str, __func: Callable[[bytes, bytes], bytes | None], arg_count: Literal[2]) -> Literal[True]: ...
@overload
def register_rpc(__name: str, __func: _RPCCallable, arg_count: int) -> Literal[True]: ...
def register_signal(__signum: int, __who: str, __handler: Callable[[int], Any]) -> None: ...
def reload() -> _TrueOrNone: ...
def request_id() -> int: ...
def route(__router_name: str, __router_args: str) -> int: ...
def rpc(__node: str | bytes, __rpc_name: bytes, *rpc_args: bytes) -> bytes | None: ...
def rpc_list() -> tuple[bytes, ...]: ...
def scrolls(__legion_name: str) -> list[bytes] | None: ...
@overload
def send(__data: bytes) -> _TrueOrNone: ...
@overload
def send(__fd: int, __data: bytes) -> _TrueOrNone: ...
def sendfile(
    __filename_or_fd: str | bytes | int | HasFileno, __chunk: int = 0, __pos: int = 0, filesize: int = 0
) -> _TrueOrNone: ...
def set_logvar(__key: str | bytes, __val: str | bytes) -> None: ...
def set_user_harakiri(__seconds: int) -> None: ...
def set_warning_message(__message: str) -> Literal[True]: ...
def setprocname(__name: str) -> None: ...
def signal(__signum: int = ..., __node: str = ...) -> None: ...
def signal_received() -> int: ...
def signal_registered(__signum: int) -> _TrueOrNone: ...
def signal_wait(__signum: int = ...) -> Literal[b""]: ...
def start_response(
    __status: str, __headers: list[tuple[str, str]], __exc_info: OptExcInfo | None = ...
) -> Callable[[bytes], None]: ...
def stop() -> _TrueOrNone: ...
def suspend() -> Literal[True]: ...
def total_requests() -> int: ...
def unlock(__lock_num: int = 0) -> None: ...
def wait_fd_read(__fd: int, __timeout: int = 0) -> Literal[b""]: ...
def wait_fd_write(__fd: int, __timeout: int = 0) -> Literal[b""]: ...
def websocket_handshake(__key: str | bytes = ..., __origin: str | bytes = ..., __proto: str | bytes = ...) -> None: ...
def websocket_recv() -> bytes: ...
def websocket_recv_nb() -> bytes: ...
def websocket_send(message: str | bytes | ReadOnlyBuffer) -> None: ...
def websocket_send_binary(message: str | bytes | ReadOnlyBuffer) -> None: ...
def worker_id() -> int: ...
def workers() -> tuple[_WorkerDict, ...] | None: ...
def sharedarea_read(__id: int, __position: int, __length: int) -> bytes: ...
def sharedarea_write(__id: int, __position: int, __value: str | bytes | ReadOnlyBuffer) -> None: ...
def sharedarea_readbyte(__id: int, __position: int) -> int: ...
def sharedarea_writebyte(__id: int, __position: int, __value: int) -> None: ...
def sharedarea_read8(__id: int, __position: int) -> int: ...
def sharedarea_write8(__id: int, __position: int, __value: int) -> None: ...
def sharedarea_readlong(__id: int, __position: int) -> int: ...
def sharedarea_writelong(__id: int, __position: int, __value: int) -> None: ...
def sharedarea_read64(__id: int, __position: int) -> int: ...
def sharedarea_write64(__id: int, __position: int, __value: int) -> None: ...
def sharedarea_read32(__id: int, __position: int) -> int: ...
def sharedarea_write32(__id: int, __position: int, __value: int) -> None: ...
def sharedarea_read16(__id: int, __position: int) -> int: ...
def sharedarea_write16(__id: int, __position: int, __value: int) -> None: ...
def sharedarea_inclong(__id: int, __position: int, __increment: int = 1) -> None: ...
def sharedarea_inc64(__id: int, __position: int, __increment: int = 1) -> None: ...
def sharedarea_inc32(__id: int, __position: int, __increment: int = 1) -> None: ...
def sharedarea_dec64(__id: int, __position: int, __decrement: int = 1) -> None: ...
def sharedarea_dec32(__id: int, __position: int, __decrement: int = 1) -> None: ...
def sharedarea_rlock(__id: int) -> None: ...
def sharedarea_wlock(__id: int) -> None: ...
def sharedarea_unlock(__id: int) -> None: ...
def sharedarea_object(__id: int) -> bytearray: ...
def sharedarea_memoryview(__id: int) -> memoryview: ...



from _typeshed import Incomplete
from collections.abc import Generator

def node_attribute_xy(G, attribute, nodes: Incomplete | None = None) -> Generator[Incomplete, None, None]: ...
def node_degree_xy(
    G, x: str = "out", y: str = "in", weight: Incomplete | None = None, nodes: Incomplete | None = None
) -> Generator[Incomplete, None, None]: ...



from networkx.exception import NetworkXError

__all__ = ["modularity", "partition_quality"]

class NotAPartition(NetworkXError):
    def __init__(self, G, collection) -> None: ...

def modularity(G, communities, weight: str = "weight", resolution: float = 1): ...
def partition_quality(G, partition): ...



from _typeshed import Incomplete

def core_number(G): ...
def k_core(G, k: Incomplete | None = None, core_number: Incomplete | None = None): ...
def k_shell(G, k: Incomplete | None = None, core_number: Incomplete | None = None): ...
def k_crust(G, k: Incomplete | None = None, core_number: Incomplete | None = None): ...
def k_corona(G, k, core_number: Incomplete | None = None): ...
def k_truss(G, k): ...
def onion_layers(G): ...



from _typeshed import Incomplete

def betweenness_centrality_subset(G, sources, targets, normalized: bool = False, weight: Incomplete | None = None): ...
def edge_betweenness_centrality_subset(G, sources, targets, normalized: bool = False, weight: Incomplete | None = None): ...



from collections.abc import Iterator, MutableSet
from re import Pattern
from typing import Any, TypedDict
from xml.etree.ElementTree import Element

from markdown.core import Markdown
from markdown.extensions import Extension
from markdown.treeprocessors import Treeprocessor

IDCOUNT_RE: Pattern[str]

class _FlatTocToken(TypedDict):
    level: int
    id: str
    name: str

class _TocToken(_FlatTocToken):
    children: list[_TocToken]

def slugify(value: str, separator: str, unicode: bool = False) -> str: ...
def slugify_unicode(value: str, separator: str) -> str: ...
def unique(id: str, ids: MutableSet[str]) -> str: ...
def get_name(el: Element) -> str: ...
def stashedHTML2text(text: str, md: Markdown, strip_entities: bool = True) -> str: ...
def unescape(text: str) -> str: ...
def nest_toc_tokens(toc_list: list[_FlatTocToken]) -> list[_TocToken]: ...

class TocTreeprocessor(Treeprocessor):
    marker: str
    title: str
    base_level: int
    slugify: Any
    sep: Any
    toc_class: Any
    title_class: str
    use_anchors: bool
    anchorlink_class: str
    use_permalinks: bool
    permalink_class: str
    permalink_title: str
    permalink_leading: bool
    header_rgx: Pattern[str]
    toc_top: int
    toc_bottom: int
    def __init__(self, md: Markdown, config: dict[str, Any]) -> None: ...
    def iterparent(self, node: Element) -> Iterator[tuple[Element, Element]]: ...
    def replace_marker(self, root: Element, elem: Element) -> None: ...
    def set_level(self, elem: Element) -> None: ...
    def add_anchor(self, c: Element, elem_id: str) -> None: ...
    def add_permalink(self, c: Element, elem_id: str) -> None: ...
    def build_toc_div(self, toc_list: list[_TocToken]) -> Element: ...
    def run(self, doc: Element) -> None: ...

class TocExtension(Extension):
    TreeProcessorClass: type[TocTreeprocessor]
    def __init__(self, **kwargs) -> None: ...
    md: Markdown
    def reset(self) -> None: ...

def makeExtension(**kwargs) -> TocExtension: ...



from _typeshed import Incomplete
from typing import Any

from boto.connection import AWSAuthConnection
from boto.exception import BotoClientError

from .bucket import Bucket

def check_lowercase_bucketname(n): ...
def assert_case_insensitive(f): ...

class _CallingFormat:
    def get_bucket_server(self, server, bucket): ...
    def build_url_base(self, connection, protocol, server, bucket, key: str = ""): ...
    def build_host(self, server, bucket): ...
    def build_auth_path(self, bucket, key: str = ""): ...
    def build_path_base(self, bucket, key: str = ""): ...

class SubdomainCallingFormat(_CallingFormat):
    def get_bucket_server(self, server, bucket): ...

class VHostCallingFormat(_CallingFormat):
    def get_bucket_server(self, server, bucket): ...

class OrdinaryCallingFormat(_CallingFormat):
    def get_bucket_server(self, server, bucket): ...
    def build_path_base(self, bucket, key: str = ""): ...

class ProtocolIndependentOrdinaryCallingFormat(OrdinaryCallingFormat):
    def build_url_base(self, connection, protocol, server, bucket, key: str = ""): ...

class Location:
    DEFAULT: str
    EU: str
    EUCentral1: str
    USWest: str
    USWest2: str
    SAEast: str
    APNortheast: str
    APSoutheast: str
    APSoutheast2: str
    CNNorth1: str

class NoHostProvided: ...
class HostRequiredError(BotoClientError): ...

class S3Connection(AWSAuthConnection):
    DefaultHost: Any
    DefaultCallingFormat: Any
    QueryString: str
    calling_format: Any
    bucket_class: type[Bucket]
    anon: Any
    def __init__(
        self,
        aws_access_key_id: Incomplete | None = None,
        aws_secret_access_key: Incomplete | None = None,
        is_secure: bool = True,
        port: Incomplete | None = None,
        proxy: Incomplete | None = None,
        proxy_port: Incomplete | None = None,
        proxy_user: Incomplete | None = None,
        proxy_pass: Incomplete | None = None,
        host: Any = ...,
        debug: int = 0,
        https_connection_factory: Incomplete | None = None,
        calling_format: Any = "boto.s3.connection.SubdomainCallingFormat",
        path: str = "/",
        provider: str = "aws",
        bucket_class: type[Bucket] = ...,
        security_token: Incomplete | None = None,
        suppress_consec_slashes: bool = True,
        anon: bool = False,
        validate_certs: Incomplete | None = None,
        profile_name: Incomplete | None = None,
    ) -> None: ...
    def __iter__(self): ...
    def __contains__(self, bucket_name): ...
    def set_bucket_class(self, bucket_class: type[Bucket]) -> None: ...
    def build_post_policy(self, expiration_time, conditions): ...
    def build_post_form_args(
        self,
        bucket_name,
        key,
        expires_in: int = 6000,
        acl: Incomplete | None = None,
        success_action_redirect: Incomplete | None = None,
        max_content_length: Incomplete | None = None,
        http_method: str = "http",
        fields: Incomplete | None = None,
        conditions: Incomplete | None = None,
        storage_class: str = "STANDARD",
        server_side_encryption: Incomplete | None = None,
    ): ...
    def generate_url_sigv4(
        self,
        expires_in,
        method,
        bucket: str = "",
        key: str = "",
        headers: dict[str, str] | None = None,
        force_http: bool = False,
        response_headers: dict[str, str] | None = None,
        version_id: Incomplete | None = None,
        iso_date: Incomplete | None = None,
    ): ...
    def generate_url(
        self,
        expires_in,
        method,
        bucket: str = "",
        key: str = "",
        headers: dict[str, str] | None = None,
        query_auth: bool = True,
        force_http: bool = False,
        response_headers: dict[str, str] | None = None,
        expires_in_absolute: bool = False,
        version_id: Incomplete | None = None,
    ): ...
    def get_all_buckets(self, headers: dict[str, str] | None = None): ...
    def get_canonical_user_id(self, headers: dict[str, str] | None = None): ...
    def get_bucket(self, bucket_name: str, validate: bool = True, headers: dict[str, str] | None = None) -> Bucket: ...
    def head_bucket(self, bucket_name, headers: dict[str, str] | None = None): ...
    def lookup(self, bucket_name, validate: bool = True, headers: dict[str, str] | None = None): ...
    def create_bucket(
        self, bucket_name, headers: dict[str, str] | None = None, location: Any = "", policy: Incomplete | None = None
    ): ...
    def delete_bucket(self, bucket, headers: dict[str, str] | None = None): ...
    def make_request(  # type: ignore[override]
        self,
        method,
        bucket: str = "",
        key: str = "",
        headers: Incomplete | None = None,
        data: str = "",
        query_args: Incomplete | None = None,
        sender: Incomplete | None = None,
        override_num_retries: Incomplete | None = None,
        retry_handler: Incomplete | None = None,
        *args,
        **kwargs,
    ): ...



from _typeshed import Incomplete
from typing import Any, overload
from typing_extensions import Self

from psycopg2._psycopg import connection, cursor

class Range:
    def __init__(
        self, lower: Incomplete | None = None, upper: Incomplete | None = None, bounds: str = "[)", empty: bool = False
    ) -> None: ...
    @property
    def lower(self): ...
    @property
    def upper(self): ...
    @property
    def isempty(self) -> bool: ...
    @property
    def lower_inf(self) -> bool: ...
    @property
    def upper_inf(self) -> bool: ...
    @property
    def lower_inc(self) -> bool: ...
    @property
    def upper_inc(self) -> bool: ...
    def __contains__(self, x) -> bool: ...
    def __bool__(self) -> bool: ...
    def __eq__(self, other: object) -> bool: ...
    def __ne__(self, other: object) -> bool: ...
    def __hash__(self) -> int: ...
    def __lt__(self, other: Range) -> bool: ...
    def __le__(self, other: Range) -> bool: ...
    def __gt__(self, other: Range) -> bool: ...
    def __ge__(self, other: Range) -> bool: ...

def register_range(
    pgrange: str, pyrange: str | type[Range], conn_or_curs: connection | cursor, globally: bool = False
) -> RangeCaster: ...

class RangeAdapter:
    name: str  # this is None here but MUST be str in subclasses
    adapted: Range
    def __init__(self, adapted: Range) -> None: ...
    def __conform__(self, proto) -> Self | None: ...
    def prepare(self, conn: connection | None) -> None: ...
    def getquoted(self) -> bytes: ...

class RangeCaster:
    adapter: type[RangeAdapter]
    range: type[Range]
    subtype_oid: Any
    typecaster: Any
    array_typecaster: Any
    def __init__(
        self,
        pgrange: str | type[RangeAdapter],
        pyrange: str | type[Range],
        oid: int,
        subtype_oid: int,
        array_oid: int | None = None,
    ) -> None: ...
    @overload
    def parse(self, s: None, cur: cursor | None = None) -> None: ...
    @overload
    def parse(self, s: str, cur: cursor | None = None) -> Range: ...

class NumericRange(Range): ...
class DateRange(Range): ...
class DateTimeRange(Range): ...
class DateTimeTZRange(Range): ...

class NumberRangeAdapter(RangeAdapter):
    def getquoted(self) -> bytes: ...

int4range_caster: RangeCaster
int8range_caster: RangeCaster
numrange_caster: RangeCaster
daterange_caster: RangeCaster
tsrange_caster: RangeCaster
tstzrange_caster: RangeCaster



from _typeshed import Incomplete

class TemplateSummarySummaryDashboards:
    openapi_types: Incomplete
    attribute_map: Incomplete
    discriminator: Incomplete
    def __init__(
        self,
        id: Incomplete | None = None,
        org_id: Incomplete | None = None,
        kind: Incomplete | None = None,
        template_meta_name: Incomplete | None = None,
        name: Incomplete | None = None,
        description: Incomplete | None = None,
        label_associations: Incomplete | None = None,
        charts: Incomplete | None = None,
        env_references: Incomplete | None = None,
    ) -> None: ...
    @property
    def id(self): ...
    @id.setter
    def id(self, id) -> None: ...
    @property
    def org_id(self): ...
    @org_id.setter
    def org_id(self, org_id) -> None: ...
    @property
    def kind(self): ...
    @kind.setter
    def kind(self, kind) -> None: ...
    @property
    def template_meta_name(self): ...
    @template_meta_name.setter
    def template_meta_name(self, template_meta_name) -> None: ...
    @property
    def name(self): ...
    @name.setter
    def name(self, name) -> None: ...
    @property
    def description(self): ...
    @description.setter
    def description(self, description) -> None: ...
    @property
    def label_associations(self): ...
    @label_associations.setter
    def label_associations(self, label_associations) -> None: ...
    @property
    def charts(self): ...
    @charts.setter
    def charts(self, charts) -> None: ...
    @property
    def env_references(self): ...
    @env_references.setter
    def env_references(self, env_references) -> None: ...
    def to_dict(self): ...
    def to_str(self): ...
    def __eq__(self, other): ...
    def __ne__(self, other): ...



from _typeshed import Incomplete

def grid_2d_graph(m, n, periodic: bool = False, create_using: Incomplete | None = None): ...
def grid_graph(dim, periodic: bool = False): ...
def hypercube_graph(n): ...
def triangular_lattice_graph(
    m, n, periodic: bool = False, with_positions: bool = True, create_using: Incomplete | None = None
): ...
def hexagonal_lattice_graph(
    m, n, periodic: bool = False, with_positions: bool = True, create_using: Incomplete | None = None
): ...



from ._hooks import HookimplMarker as HookimplMarker, HookspecMarker as HookspecMarker
from ._manager import PluginManager as PluginManager, PluginValidationError as PluginValidationError
from ._result import HookCallError as HookCallError

__version__: str



from _typeshed import Incomplete

def modularity_matrix(G, nodelist: Incomplete | None = None, weight: Incomplete | None = None): ...
def directed_modularity_matrix(G, nodelist: Incomplete | None = None, weight: Incomplete | None = None): ...



from _typeshed import Incomplete
from typing import ClassVar, Literal

from ._imaging import _PixelAccessor
from .ImageFile import ImageFile

COMPRESSION: Incomplete
PAD: Incomplete

def i(c): ...
def dump(c) -> None: ...

class IptcImageFile(ImageFile):
    format: ClassVar[Literal["IPTC"]]
    format_description: ClassVar[str]
    def getint(self, key): ...
    def field(self): ...
    im: Incomplete
    def load(self) -> _PixelAccessor: ...

def getiptcinfo(im): ...



from _typeshed import Incomplete
from collections.abc import Generator

def attracting_components(G) -> Generator[Incomplete, None, None]: ...
def number_attracting_components(G): ...
def is_attracting_component(G): ...



from _typeshed import Incomplete
from collections.abc import Generator

def asyn_lpa_communities(
    G, weight: Incomplete | None = None, seed: Incomplete | None = None
) -> Generator[Incomplete, Incomplete, None]: ...
def label_propagation_communities(G): ...



from _typeshed import Incomplete
from collections.abc import Generator

def triadic_census(G, nodelist: Incomplete | None = None): ...
def is_triad(G): ...
def all_triplets(G): ...
def all_triads(G) -> Generator[Incomplete, None, None]: ...
def triads_by_type(G): ...
def triad_type(G): ...
def random_triad(G, seed: Incomplete | None = None): ...



from _typeshed import Incomplete

def resource_allocation_index(G, ebunch: Incomplete | None = None): ...
def jaccard_coefficient(G, ebunch: Incomplete | None = None): ...
def adamic_adar_index(G, ebunch: Incomplete | None = None): ...
def common_neighbor_centrality(G, ebunch: Incomplete | None = None, alpha: float = 0.8): ...
def preferential_attachment(G, ebunch: Incomplete | None = None): ...
def cn_soundarajan_hopcroft(G, ebunch: Incomplete | None = None, community: str = "community"): ...
def ra_index_soundarajan_hopcroft(G, ebunch: Incomplete | None = None, community: str = "community"): ...
def within_inter_cluster(G, ebunch: Incomplete | None = None, delta: float = 0.001, community: str = "community"): ...



from _typeshed import Incomplete

def is_valid_joint_degree(joint_degrees): ...
def joint_degree_graph(joint_degrees, seed: Incomplete | None = None): ...
def is_valid_directed_joint_degree(in_degrees, out_degrees, nkk): ...
def directed_joint_degree_graph(in_degrees, out_degrees, nkk, seed: Incomplete | None = None): ...



from _typeshed import Incomplete

from .edmondskarp import edmonds_karp

__all__ = ["gomory_hu_tree"]

default_flow_func = edmonds_karp

def gomory_hu_tree(G, capacity: str = "capacity", flow_func: Incomplete | None = None): ...



from _typeshed import ExcInfo
from collections.abc import Callable
from typing import Generic, TypeVar

_T = TypeVar("_T")

class HookCallError(Exception): ...

class _Result(Generic[_T]):
    def __init__(self, result: _T | None, exception: BaseException | None) -> None: ...
    @property
    def excinfo(self) -> ExcInfo | None: ...
    @property
    def exception(self) -> BaseException | None: ...
    @classmethod
    def from_call(cls, func: Callable[[], _T]) -> _Result[_T]: ...
    def force_result(self, result: _T) -> None: ...
    def force_exception(self, exception: BaseException) -> None: ...
    def get_result(self) -> _T: ...



# Verify that ImageTK images are valid to pass to TK code.
from __future__ import annotations

import tkinter

from PIL import ImageTk

photo = ImageTk.PhotoImage()
bitmap = ImageTk.BitmapImage()

tkinter.Label(image=photo)
tkinter.Label(image=bitmap)

tkinter.Label().configure(image=photo)
tkinter.Label().configure(image=bitmap)



from _typeshed import Incomplete
from collections.abc import Iterable
from dataclasses import dataclass
from io import BytesIO
from typing import Literal

from PIL import Image

from .drawing import DeviceGray, DeviceRGB
from .enums import Align, TableBordersLayout, TableCellFillMode, VAlign, WrapMode
from .fonts import FontFace
from .fpdf import FPDF
from .util import Padding

DEFAULT_HEADINGS_STYLE: FontFace

class Table:
    rows: list[Row]

    def __init__(
        self,
        fpdf: FPDF,
        rows: Iterable[str] = (),
        *,
        align: str | Align = "CENTER",
        v_align: str | VAlign = "MIDDLE",
        borders_layout: str | TableBordersLayout = ...,
        cell_fill_color: int | tuple[Incomplete, ...] | DeviceGray | DeviceRGB | None = None,
        cell_fill_mode: str | TableCellFillMode = ...,
        col_widths: int | tuple[int, ...] | None = None,
        first_row_as_headings: bool = True,
        gutter_height: float = 0,
        gutter_width: float = 0,
        headings_style: FontFace = ...,
        line_height: int | None = None,
        markdown: bool = False,
        text_align: str | Align = "JUSTIFY",
        width: int | None = None,
        wrapmode: WrapMode = ...,
        padding: float | Padding | None = None,
        outer_border_width: float | None = None,
        num_heading_rows: int = 1,
    ) -> None: ...
    def row(self, cells: Iterable[str] = (), style: FontFace | None = None) -> Row: ...
    def render(self) -> None: ...
    def get_cell_border(self, i, j) -> str | Literal[0, 1]: ...

class Row:
    cells: list[Cell]
    style: FontFace
    def __init__(self, table: Table, style: FontFace | None = None) -> None: ...
    @property
    def cols_count(self) -> int: ...
    @property
    def column_indices(self): ...
    def cell(
        self,
        text: str = "",
        align: str | Align | None = None,
        v_align: str | VAlign | None = None,
        style: FontFace | None = None,
        img: str | Image.Image | BytesIO | None = None,
        img_fill_width: bool = False,
        colspan: int = 1,
        padding: tuple[float, ...] | None = None,
        link: str | int | None = None,
    ) -> Cell: ...

@dataclass
class Cell:
    text: str
    align: str | Align | None
    v_align: str | VAlign | None
    style: FontFace | None
    img: str | None
    img_fill_width: bool
    colspan: int
    padding: int | tuple[float, ...] | None
    link: str | int | None

    def write(self, text, align: Incomplete | None = None): ...

@dataclass(frozen=True)
class RowLayoutInfo:
    height: int
    triggers_page_jump: bool
    rendered_height: dict[Incomplete, Incomplete]

def draw_box_borders(pdf: FPDF, x1, y1, x2, y2, border: str | Literal[0, 1], fill_color: Incomplete | None = None) -> None: ...



from _typeshed import Incomplete

def uniform_random_intersection_graph(n, m, p, seed: Incomplete | None = None): ...
def k_random_intersection_graph(n, m, k, seed: Incomplete | None = None): ...
def general_random_intersection_graph(n, m, p, seed: Incomplete | None = None): ...



from _typeshed import Incomplete
from collections.abc import Callable
from re import Pattern
from typing import AnyStr, Protocol

PY3: bool
text_type: Callable[[Incomplete], Incomplete]

class _NullCoder:
    @staticmethod
    def encode(b: str, final: bool = False): ...
    @staticmethod
    def decode(b: str, final: bool = False): ...

class _Logfile(Protocol):
    def write(self, __s) -> object: ...
    def flush(self) -> object: ...

class SpawnBase:
    encoding: Incomplete
    pid: Incomplete
    flag_eof: bool
    stdin: Incomplete
    stdout: Incomplete
    stderr: Incomplete
    searcher: Incomplete
    ignorecase: bool
    before: Incomplete
    after: Incomplete
    match: Incomplete
    match_index: Incomplete
    terminated: bool
    exitstatus: Incomplete
    signalstatus: Incomplete
    status: Incomplete
    child_fd: int
    timeout: Incomplete
    delimiter: Incomplete
    logfile: _Logfile
    logfile_read: _Logfile
    logfile_send: _Logfile
    maxread: Incomplete
    searchwindowsize: Incomplete
    delaybeforesend: float | None
    delayafterclose: float
    delayafterterminate: float
    delayafterread: float
    softspace: bool
    name: Incomplete
    closed: bool
    codec_errors: Incomplete
    string_type: Incomplete
    buffer_type: Incomplete
    crlf: bytes
    allowed_string_types: Incomplete
    linesep: Incomplete
    write_to_stdout: Incomplete
    async_pw_transport: Incomplete
    def __init__(
        self,
        timeout: int = 30,
        maxread: int = 2000,
        searchwindowsize: Incomplete | None = None,
        logfile: _Logfile | None = None,
        encoding: Incomplete | None = None,
        codec_errors: str = "strict",
    ) -> None: ...
    buffer: Incomplete
    def read_nonblocking(self, size: int = 1, timeout: int | None = None) -> bytes: ...
    def compile_pattern_list(self, patterns) -> list[Pattern[AnyStr]]: ...
    def expect(self, pattern, timeout: int = -1, searchwindowsize: int = -1, async_: bool = False, **kw) -> int: ...
    def expect_list(self, pattern_list, timeout: int = -1, searchwindowsize: int = -1, async_: bool = False, **kw) -> int: ...
    def expect_exact(self, pattern_list, timeout: int = -1, searchwindowsize: int = -1, async_: bool = False, **kw) -> int: ...
    def expect_loop(self, searcher, timeout: int = -1, searchwindowsize: int = -1) -> int: ...
    def read(self, size: int = -1) -> bytes: ...
    def readline(self, size: int = -1) -> bytes: ...
    def __iter__(self): ...
    def readlines(self, sizehint: int = -1) -> list[str]: ...
    def fileno(self): ...
    def flush(self) -> None: ...
    def isatty(self): ...
    def __enter__(self): ...
    def __exit__(self, etype, evalue, tb) -> None: ...



from _typeshed import Incomplete

from pexpect import ExceptionPexpect, spawn

from .spawnbase import _Logfile

class ExceptionPxssh(ExceptionPexpect): ...

class pxssh(spawn):
    name: str
    UNIQUE_PROMPT: str
    PROMPT: Incomplete
    PROMPT_SET_SH: str
    PROMPT_SET_CSH: str
    SSH_OPTS: Incomplete
    force_password: bool
    debug_command_string: Incomplete
    options: Incomplete
    def __init__(
        self,
        timeout: int = 30,
        maxread: int = 2000,
        searchwindowsize: Incomplete | None = None,
        logfile: _Logfile | None = None,
        cwd: Incomplete | None = None,
        env: Incomplete | None = None,
        ignore_sighup: bool = True,
        echo: bool = True,
        options={},
        encoding: Incomplete | None = None,
        codec_errors: str = "strict",
        debug_command_string: bool = False,
        use_poll: bool = False,
    ) -> None: ...
    def levenshtein_distance(self, a, b): ...
    def try_read_prompt(self, timeout_multiplier): ...
    def sync_original_prompt(self, sync_multiplier: float = 1.0): ...
    def login(
        self,
        server,
        username: Incomplete | None = None,
        password: str = "",
        terminal_type: str = "ansi",
        original_prompt: str = "[#$]",
        login_timeout: int = 10,
        port: Incomplete | None = None,
        auto_prompt_reset: bool = True,
        ssh_key: Incomplete | None = None,
        quiet: bool = True,
        sync_multiplier: int = 1,
        check_local_ip: bool = True,
        password_regex: str = "(?i)(?:password:)|(?:passphrase for key)",
        ssh_tunnels={},
        spawn_local_ssh: bool = True,
        sync_original_prompt: bool = True,
        ssh_config: Incomplete | None = None,
        cmd: str = "ssh",
    ): ...
    def logout(self) -> None: ...
    def prompt(self, timeout: int = -1): ...
    def set_unique_prompt(self): ...



from _typeshed import Incomplete

from pexpect.spawnbase import _Logfile

def run(
    command,
    timeout: int = 30,
    withexitstatus: bool = False,
    events: Incomplete | None = None,
    extra_args: Incomplete | None = None,
    logfile: _Logfile | None = None,
    cwd: Incomplete | None = None,
    env: Incomplete | None = None,
    **kwargs,
): ...
def runu(
    command,
    timeout: int = 30,
    withexitstatus: bool = False,
    events: Incomplete | None = None,
    extra_args: Incomplete | None = None,
    logfile: _Logfile | None = None,
    cwd: Incomplete | None = None,
    env: Incomplete | None = None,
    **kwargs,
): ...



from networkx.exception import NetworkXException

class NotATree(NetworkXException): ...

def to_nested_tuple(T, root, canonical_form: bool = False): ...
def from_nested_tuple(sequence, sensible_relabeling: bool = False): ...
def to_prufer_sequence(T): ...
def from_prufer_sequence(sequence): ...



from _typeshed import Incomplete

from networkx.classes.graph import Graph, _Node

def descendants(G: Graph[_Node], source: _Node) -> set[_Node]: ...
def ancestors(G: Graph[_Node], source: _Node) -> set[_Node]: ...
def is_directed_acyclic_graph(G): ...
def topological_sort(G) -> None: ...
def lexicographical_topological_sort(G, key: Incomplete | None = None): ...
def all_topological_sorts(G) -> None: ...
def is_aperiodic(G): ...
def transitive_closure(G, reflexive: bool = False): ...
def transitive_reduction(G): ...
def antichains(G, topo_order: Incomplete | None = None) -> None: ...
def dag_longest_path(G, weight: str = "weight", default_weight: int = 1, topo_order: Incomplete | None = None): ...
def dag_longest_path_length(G, weight: str = "weight", default_weight: int = 1): ...
def dag_to_branching(G): ...



from _typeshed import Incomplete
from collections.abc import Iterable, Mapping
from types import TracebackType
from typing_extensions import Self, TypeAlias
from urllib.parse import ParseResult, SplitResult

from requests.auth import AuthBase
from requests.models import Response
from requests.sessions import _Timeout
from requests.structures import CaseInsensitiveDict

from .lib.url import URL
from .objects import Calendar, DAVObject, Principal

_Element: TypeAlias = Incomplete  # actually lxml.etree._Element

class DAVResponse:
    reason: str
    tree: _Element | None
    status: int
    headers: CaseInsensitiveDict[str]
    objects: dict[str, dict[str, str]]  # only defined after call to find_objects_and_props()
    huge_tree: bool
    def __init__(self, response: Response, davclient: DAVClient | None = None) -> None: ...
    @property
    def raw(self) -> str: ...
    def validate_status(self, status: str) -> None: ...
    def find_objects_and_props(self) -> None: ...
    def expand_simple_props(
        self, props: Iterable[Incomplete] = [], multi_value_props: Iterable[Incomplete] = [], xpath: str | None = None
    ) -> dict[str, dict[str, str]]: ...

class DAVClient:
    proxy: str | None
    url: URL
    headers: dict[str, str]
    username: str | None
    password: str | None
    auth: AuthBase | None
    timeout: _Timeout | None
    ssl_verify_cert: bool | str
    ssl_cert: str | tuple[str, str] | None
    huge_tree: bool
    def __init__(
        self,
        url: str,
        proxy: str | None = None,
        username: str | None = None,
        password: str | None = None,
        auth: AuthBase | None = None,
        timeout: _Timeout | None = None,
        ssl_verify_cert: bool | str = True,
        ssl_cert: str | tuple[str, str] | None = None,
        headers: dict[str, str] = {},
        huge_tree: bool = False,
    ) -> None: ...
    def __enter__(self) -> Self: ...
    def __exit__(
        self, exc_type: type[BaseException] | None, exc_value: BaseException | None, traceback: TracebackType | None
    ) -> None: ...
    def principal(self, *, url: str | ParseResult | SplitResult | URL | None = ...) -> Principal: ...
    def calendar(
        self,
        url: str | ParseResult | SplitResult | URL | None = ...,
        parent: DAVObject | None = ...,
        name: str | None = ...,
        id: str | None = ...,
        props: Mapping[Incomplete, Incomplete] = ...,
        **extra: Incomplete,
    ) -> Calendar: ...
    def check_dav_support(self) -> str | None: ...
    def check_cdav_support(self) -> bool: ...
    def check_scheduling_support(self) -> bool: ...
    def propfind(self, url: str | None = None, props: str = "", depth: int = 0) -> DAVResponse: ...
    def proppatch(self, url: str, body: str, dummy: None = None) -> DAVResponse: ...
    def report(self, url: str, query: str = "", depth: int = 0) -> DAVResponse: ...
    def mkcol(self, url: str, body: str, dummy: None = None) -> DAVResponse: ...
    def mkcalendar(self, url: str, body: str = "", dummy: None = None) -> DAVResponse: ...
    def put(self, url: str, body: str, headers: Mapping[str, str] = {}) -> DAVResponse: ...
    def post(self, url: str, body: str, headers: Mapping[str, str] = {}) -> DAVResponse: ...
    def delete(self, url: str) -> DAVResponse: ...
    def options(self, url: str) -> DAVResponse: ...
    def request(self, url: str, method: str = "GET", body: str = "", headers: Mapping[str, str] = {}) -> DAVResponse: ...



from _typeshed import Incomplete

def randomized_partitioning(G, seed: Incomplete | None = None, p: float = 0.5, weight: Incomplete | None = None): ...
def one_exchange(G, initial_cut: Incomplete | None = None, seed: Incomplete | None = None, weight: Incomplete | None = None): ...



import sys
from _typeshed import FileDescriptorLike, ReadOnlyBuffer, WriteableBuffer
from typing import Any, Literal, overload
from typing_extensions import Buffer

if sys.platform != "win32":
    FASYNC: int
    FD_CLOEXEC: int
    F_DUPFD: int
    F_DUPFD_CLOEXEC: int
    F_GETFD: int
    F_GETFL: int
    F_GETLK: int
    F_GETOWN: int
    F_RDLCK: int
    F_SETFD: int
    F_SETFL: int
    F_SETLK: int
    F_SETLKW: int
    F_SETOWN: int
    F_UNLCK: int
    F_WRLCK: int

    F_GETLEASE: int
    F_SETLEASE: int
    if sys.platform == "darwin":
        F_FULLFSYNC: int
        F_NOCACHE: int
        if sys.version_info >= (3, 9):
            F_GETPATH: int
    if sys.platform == "linux":
        F_SETLKW64: int
        F_SETSIG: int
        F_SHLCK: int
        F_SETLK64: int
        F_GETSIG: int
        F_NOTIFY: int
        F_EXLCK: int
        F_GETLK64: int
        F_ADD_SEALS: int
        F_GET_SEALS: int
        F_SEAL_GROW: int
        F_SEAL_SEAL: int
        F_SEAL_SHRINK: int
        F_SEAL_WRITE: int
        if sys.version_info >= (3, 9):
            F_OFD_GETLK: int
            F_OFD_SETLK: int
            F_OFD_SETLKW: int
        if sys.version_info >= (3, 10):
            F_GETPIPE_SZ: int
            F_SETPIPE_SZ: int

        DN_ACCESS: int
        DN_ATTRIB: int
        DN_CREATE: int
        DN_DELETE: int
        DN_MODIFY: int
        DN_MULTISHOT: int
        DN_RENAME: int

    LOCK_EX: int
    LOCK_NB: int
    LOCK_SH: int
    LOCK_UN: int
    if sys.platform == "linux":
        LOCK_MAND: int
        LOCK_READ: int
        LOCK_RW: int
        LOCK_WRITE: int

    # These are highly problematic, they might be present or not, depends on the specific OS.
    if sys.platform == "linux":
        I_ATMARK: int
        I_CANPUT: int
        I_CKBAND: int
        I_FDINSERT: int
        I_FIND: int
        I_FLUSH: int
        I_FLUSHBAND: int
        I_GETBAND: int
        I_GETCLTIME: int
        I_GETSIG: int
        I_GRDOPT: int
        I_GWROPT: int
        I_LINK: int
        I_LIST: int
        I_LOOK: int
        I_NREAD: int
        I_PEEK: int
        I_PLINK: int
        I_POP: int
        I_PUNLINK: int
        I_PUSH: int
        I_RECVFD: int
        I_SENDFD: int
        I_SETCLTIME: int
        I_SETSIG: int
        I_SRDOPT: int
        I_STR: int
        I_SWROPT: int
        I_UNLINK: int

    if sys.version_info >= (3, 12) and sys.platform == "linux":
        FICLONE: int
        FICLONERANGE: int

    @overload
    def fcntl(__fd: FileDescriptorLike, __cmd: int, __arg: int = 0) -> int: ...
    @overload
    def fcntl(__fd: FileDescriptorLike, __cmd: int, __arg: str | ReadOnlyBuffer) -> bytes: ...
    # If arg is an int, return int
    @overload
    def ioctl(__fd: FileDescriptorLike, __request: int, __arg: int = 0, __mutate_flag: bool = True) -> int: ...
    # The return type works as follows:
    # - If arg is a read-write buffer, return int if mutate_flag is True, otherwise bytes
    # - If arg is a read-only buffer, return bytes (and ignore the value of mutate_flag)
    # We can't represent that precisely as we can't distinguish between read-write and read-only
    # buffers, so we add overloads for a few unambiguous cases and use Any for the rest.
    @overload
    def ioctl(__fd: FileDescriptorLike, __request: int, __arg: bytes, __mutate_flag: bool = True) -> bytes: ...
    @overload
    def ioctl(__fd: FileDescriptorLike, __request: int, __arg: WriteableBuffer, __mutate_flag: Literal[False]) -> bytes: ...
    @overload
    def ioctl(__fd: FileDescriptorLike, __request: int, __arg: Buffer, __mutate_flag: bool = True) -> Any: ...
    def flock(__fd: FileDescriptorLike, __operation: int) -> None: ...
    def lockf(__fd: FileDescriptorLike, __cmd: int, __len: int = 0, __start: int = 0, __whence: int = 0) -> Any: ...



from collections.abc import Callable, Generator, Mapping, Sequence
from types import ModuleType
from typing import Any, TypedDict, TypeVar, overload
from typing_extensions import TypeAlias

from ._result import _Result

_T = TypeVar("_T")
_F = TypeVar("_F", bound=Callable[..., object])
_Namespace: TypeAlias = ModuleType | type
_Plugin: TypeAlias = object
_HookExec: TypeAlias = Callable[[str, Sequence[HookImpl], Mapping[str, object], bool], object | list[object]]
_HookImplFunction: TypeAlias = Callable[..., _T | Generator[None, _Result[_T], None]]

class _HookSpecOpts(TypedDict):
    firstresult: bool
    historic: bool
    warn_on_impl: Warning | None

class _HookImplOpts(TypedDict):
    wrapper: bool
    hookwrapper: bool
    optionalhook: bool
    tryfirst: bool
    trylast: bool
    specname: str | None

class HookspecMarker:
    project_name: str
    def __init__(self, project_name: str) -> None: ...
    @overload
    def __call__(
        self, function: _F, firstresult: bool = False, historic: bool = False, warn_on_impl: Warning | None = None
    ) -> _F: ...
    @overload
    def __call__(
        self, function: None = None, firstresult: bool = False, historic: bool = False, warn_on_impl: Warning | None = None
    ) -> Callable[[_F], _F]: ...

class HookimplMarker:
    project_name: str
    def __init__(self, project_name: str) -> None: ...
    @overload
    def __call__(
        self,
        function: _F,
        hookwrapper: bool = False,
        optionalhook: bool = False,
        tryfirst: bool = False,
        trylast: bool = False,
        specname: str | None = None,
        wrapper: bool = False,
    ) -> _F: ...
    @overload
    def __call__(
        self,
        function: None = None,
        hookwrapper: bool = False,
        optionalhook: bool = False,
        tryfirst: bool = False,
        trylast: bool = False,
        specname: str | None = None,
        wrapper: bool = False,
    ) -> Callable[[_F], _F]: ...

def normalize_hookimpl_opts(opts: _HookImplOpts) -> None: ...
def varnames(func: object) -> tuple[tuple[str, ...], tuple[str, ...]]: ...

class _HookRelay:
    def __getattr__(self, name: str) -> _HookCaller: ...

class _HookCaller:
    name: str
    spec: HookSpec | None
    def __init__(
        self,
        name: str,
        hook_execute: _HookExec,
        specmodule_or_class: _Namespace | None = None,
        spec_opts: _HookSpecOpts | None = None,
    ) -> None: ...
    def has_spec(self) -> bool: ...
    def set_specification(self, specmodule_or_class: _Namespace, spec_opts: _HookSpecOpts) -> None: ...
    def is_historic(self) -> bool: ...
    def get_hookimpls(self) -> list[HookImpl]: ...
    def __call__(self, **kwargs: object) -> Any: ...
    def call_historic(
        self, result_callback: Callable[[Any], None] | None = None, kwargs: Mapping[str, object] | None = None
    ) -> None: ...
    def call_extra(self, methods: Sequence[Callable[..., object]], kwargs: Mapping[str, object]) -> Any: ...

class HookImpl:
    function: _HookImplFunction[object]
    plugin: _Plugin
    opts: _HookImplOpts
    plugin_name: str
    wrapper: bool
    hookwrapper: bool
    optionalhook: bool
    tryfirst: bool
    trylast: bool
    argnames: tuple[str, ...]
    kwargnames: tuple[str, ...]
    def __init__(
        self, plugin: _Plugin, plugin_name: str, function: _HookImplFunction[object], hook_impl_opts: _HookImplOpts
    ) -> None: ...

class HookSpec:
    namespace: _Namespace
    function: Callable[..., object]
    name: str
    opts: _HookSpecOpts
    warn_on_impl: Warning | None
    argnames: tuple[str, ...]
    kwargnames: tuple[str, ...]
    def __init__(self, namespace: _Namespace, name: str, opts: _HookSpecOpts) -> None: ...



from collections.abc import Iterable
from typing import ClassVar, Literal, NoReturn

class Quitter:
    name: str
    eof: str
    def __init__(self, name: str, eof: str) -> None: ...
    def __call__(self, code: int | None = None) -> NoReturn: ...

class _Printer:
    MAXLINES: ClassVar[Literal[23]]
    def __init__(self, name: str, data: str, files: Iterable[str] = (), dirs: Iterable[str] = ()) -> None: ...
    def __call__(self) -> None: ...

class _Helper:
    def __call__(self, request: object) -> None: ...



from _typeshed import Incomplete
from _typeshed.wsgi import WSGIApplication
from typing import ClassVar, overload
from typing_extensions import Self

from webob.request import Request

class wsgify:
    RequestClass: ClassVar[type[Request]]
    func: Incomplete
    args: Incomplete
    kwargs: Incomplete
    middleware_wraps: Incomplete
    def __init__(
        self,
        func: Incomplete | None = None,
        RequestClass: Incomplete | None = None,
        args=...,
        kwargs: Incomplete | None = None,
        middleware_wraps: Incomplete | None = None,
    ) -> None: ...
    @overload
    def __get__(self, obj: None, type: type | None = None) -> Self: ...
    @overload
    def __get__(self, obj: object, type: type | None = None) -> WSGIApplication: ...
    def __call__(self, req, *args, **kw): ...
    def get(self, url, **kw): ...
    def post(self, url, POST: Incomplete | None = None, **kw): ...
    def request(self, url, **kw): ...
    def call_func(self, req, *args, **kwargs): ...
    def clone(self, func: Incomplete | None = None, **kw): ...
    @property
    def undecorated(self): ...
    @classmethod
    def middleware(cls, middle_func: Incomplete | None = None, app: Incomplete | None = None, **kw): ...

class _UnboundMiddleware:
    wrapper_class: Incomplete
    app: Incomplete
    kw: Incomplete
    def __init__(self, wrapper_class, app, kw) -> None: ...
    def __call__(self, func, app: Incomplete | None = None): ...

class _MiddlewareFactory:
    wrapper_class: Incomplete
    middleware: Incomplete
    kw: Incomplete
    def __init__(self, wrapper_class, middleware, kw) -> None: ...
    def __call__(self, app: Incomplete | None = None, **config): ...



from _typeshed import Incomplete

def astar_path(G, source, target, heuristic: Incomplete | None = None, weight: str = "weight"): ...
def astar_path_length(G, source, target, heuristic: Incomplete | None = None, weight: str = "weight"): ...



from _typeshed import Incomplete

def betweenness_centrality(
    G,
    k: Incomplete | None = None,
    normalized: bool = True,
    weight: Incomplete | None = None,
    endpoints: bool = False,
    seed: Incomplete | None = None,
): ...
def edge_betweenness_centrality(
    G, k: Incomplete | None = None, normalized: bool = True, weight: Incomplete | None = None, seed: Incomplete | None = None
): ...



from _typeshed import Incomplete

from networkx.algorithms.flow import edmonds_karp

__all__ = ["k_components"]

default_flow_func = edmonds_karp

def k_components(G, flow_func: Incomplete | None = None): ...



from _typeshed import Incomplete

def write_dot(G, path) -> None: ...
def read_dot(path): ...
def from_pydot(P): ...
def to_pydot(N): ...
def graphviz_layout(G, prog: str = "neato", root: Incomplete | None = None): ...
def pydot_layout(G, prog: str = "neato", root: Incomplete | None = None): ...



from _typeshed import Incomplete
from collections.abc import Generator

def nonisomorphic_trees(order, create: str = "graph") -> Generator[Incomplete, None, None]: ...
def number_of_nonisomorphic_trees(order): ...



def maximal_matching(G): ...
def is_matching(G, matching): ...
def is_maximal_matching(G, matching): ...
def is_perfect_matching(G, matching): ...
def min_weight_matching(G, weight: str = "weight"): ...
def max_weight_matching(G, maxcardinality: bool = False, weight: str = "weight"): ...



from collections.abc import Callable
from typing import Any
from typing_extensions import TypeAlias

_Writer: TypeAlias = Callable[[str], object]
_Processor: TypeAlias = Callable[[tuple[str, ...], tuple[Any, ...]], None]

class TagTracer:
    indent: int
    def __init__(self) -> None: ...
    def get(self, name: str) -> TagTracerSub: ...
    def setwriter(self, writer: _Writer | None) -> None: ...
    def setprocessor(self, tags: str | tuple[str, ...], processor: _Processor) -> None: ...

class TagTracerSub:
    root: TagTracer
    tags: tuple[str, ...]
    def __init__(self, root: TagTracer, tags: tuple[str, ...]) -> None: ...
    def __call__(self, *args: object) -> None: ...
    def get(self, name: str) -> TagTracerSub: ...



from _typeshed import Incomplete
from collections.abc import Callable, Iterable, Mapping, Sequence
from importlib.metadata import Distribution
from typing import Any
from typing_extensions import TypeAlias

from ._hooks import (
    HookImpl as HookImpl,
    HookSpec as HookSpec,
    _HookCaller,
    _HookImplOpts,
    _HookRelay,
    _HookSpecOpts,
    _Namespace,
    _Plugin,
    normalize_hookimpl_opts as normalize_hookimpl_opts,
)
from ._result import _Result as _Result
from ._tracing import TagTracerSub

_BeforeTrace: TypeAlias = Callable[[str, Sequence[HookImpl], Mapping[str, Any]], None]
_AfterTrace: TypeAlias = Callable[[_Result[Any], str, Sequence[HookImpl], Mapping[str, Any]], None]

class PluginValidationError(Exception):
    plugin: _Plugin
    def __init__(self, plugin: _Plugin, message: str) -> None: ...

class DistFacade:
    def __init__(self, dist: Distribution) -> None: ...
    @property
    def project_name(self) -> str: ...
    def __getattr__(self, attr: str, default: Incomplete | None = None) -> Any: ...
    def __dir__(self) -> list[str]: ...

class PluginManager:
    project_name: str
    trace: TagTracerSub
    hook: _HookRelay
    def __init__(self, project_name: str) -> None: ...
    def register(self, plugin: _Plugin, name: str | None = None) -> str | None: ...
    def parse_hookimpl_opts(self, plugin: _Plugin, name: str) -> _HookImplOpts | None: ...
    def unregister(self, plugin: _Plugin | None = None, name: str | None = None) -> _Plugin: ...
    def set_blocked(self, name: str) -> None: ...
    def is_blocked(self, name: str) -> bool: ...
    def add_hookspecs(self, module_or_class: _Namespace) -> None: ...
    def parse_hookspec_opts(self, module_or_class: _Namespace, name: str) -> _HookSpecOpts | None: ...
    def get_plugins(self) -> set[Any]: ...
    def is_registered(self, plugin: _Plugin) -> bool: ...
    def get_canonical_name(self, plugin: _Plugin) -> str: ...
    def get_plugin(self, name: str) -> Any | None: ...
    def has_plugin(self, name: str) -> bool: ...
    def get_name(self, plugin: _Plugin) -> str | None: ...
    def check_pending(self) -> None: ...
    def load_setuptools_entrypoints(self, group: str, name: str | None = None) -> int: ...
    def list_plugin_distinfo(self) -> list[tuple[_Plugin, DistFacade]]: ...
    def list_name_plugin(self) -> list[tuple[str, _Plugin]]: ...
    def get_hookcallers(self, plugin: _Plugin) -> list[_HookCaller] | None: ...
    def add_hookcall_monitoring(self, before: _BeforeTrace, after: _AfterTrace) -> Callable[[], None]: ...
    def enable_tracing(self) -> Callable[[], None]: ...
    def subset_hook_caller(self, name: str, remove_plugins: Iterable[_Plugin]) -> _HookCaller: ...



from _typeshed import Incomplete

def margulis_gabber_galil_graph(n, create_using: Incomplete | None = None): ...
def chordal_cycle_graph(p, create_using: Incomplete | None = None): ...
def paley_graph(p, create_using: Incomplete | None = None): ...



from _typeshed import Incomplete

def floyd_warshall_numpy(G, nodelist: Incomplete | None = None, weight: str = "weight"): ...
def floyd_warshall_predecessor_and_distance(G, weight: str = "weight"): ...
def reconstruct_path(source, target, predecessors): ...
def floyd_warshall(G, weight: str = "weight"): ...



from . import (
    backends as backends,
    coreviews as coreviews,
    filters as filters,
    graphviews as graphviews,
    reportviews as reportviews,
)
from .digraph import DiGraph as DiGraph
from .function import *
from .graph import Graph as Graph
from .multidigraph import MultiDiGraph as MultiDiGraph
from .multigraph import MultiGraph as MultiGraph



from _typeshed import Incomplete

def latapy_clustering(G, nodes: Incomplete | None = None, mode: str = "dot"): ...

clustering = latapy_clustering

def average_clustering(G, nodes: Incomplete | None = None, mode: str = "dot"): ...
def robins_alexander_clustering(G): ...
